/*
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 * JVending
 * Copyright (c) 2005 Shane Isbell.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by
 *        JVending (http://jvending.sourceforge.net/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "JVending" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact shane.isbell@gmail.com.
 *
 * 5. Products derived from this software may not be called "JVending"
 *    nor may "JVending" appear in their names without prior written
 *    permission of Shane Isbell.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 *
 */

package org.jvending.registry.hibernate.impl;

import org.kxml2.io.KXmlParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParser;

import java.io.*;
import java.util.*;
import java.util.logging.Logger;

import net.sf.hibernate.SessionFactory;
import net.sf.hibernate.HibernateException;
import net.sf.hibernate.cfg.Configuration;

import org.jvending.registry.RepositoryRegistry;
import org.jvending.registry.hibernate.Hibernate2Repository;
import org.jvending.registry.hibernate.HibernateRepositoryAdaptor;
import org.jvending.registry.hibernate.HibernateRepositoryAdaptee;

/**
 * This class provides services for accessing hibernate2 session factories. The following code shows how to get
 * hibernate sessions from multiple DBs.
 * <pre>
        repositoryRegistry.loadFromFile("/home/name/registry-config.xml");

        Hibernate2Repository hsqlRepository = (Hibernate2Repository) repositoryRegistry.find("hibernate-hsqldb");
        SessionFactory hsqlPeerFactory = hsqlRepository.getSessionFactoryByName("peer");
        Session peerSession = hsqlPeerFactory.openSession();

        Hibernate2Repository mysqlRepository = (Hibernate2Repository) repositoryRegistry.find("hibernate-mysql");
        SessionFactory mysqlUserFactory = mysqlRepository.getSessionFactoryByName("user");
        Session userSession = mysqlUserFactory.openSession();
</pre>

*A typical mapping path looks like
<pre>
  registry-config.xml   -> {connections-hsqldb.xml, connections-mysql.xml}
                        -> {hibernate-hsqldb.cfg, hibernate-mysql.cfg}
                        -> {peer.hbm[hsqldb], catalog.hbm[hsqldb], user[mysql], ...}
</pre>
* This allows a single property file to load up connections to multiple databases, each, in turn, mapping to multiple
* database instances (schemas).
* <p>registry-config.xml</p>
 <pre>
 &lt;registry-config&gt;
   &lt;repositories&gt;
      &lt;repository&gt;
         &lt;repository-name&gt;hibernate-hsqldb&lt;/repository-name&gt;
         &lt;repository-class&gt;org.jvending.util.registry.Hibernate2RepositoryImpl&lt;/repository-class&gt;
         &lt;repository-config&gt;${basedir}/src/test/resources/hibernate2/connections-hsqldb.xml&lt;/repository-config&gt;
      &lt;/repository&gt;

      &lt;repository&gt;
         &lt;repository-name&gt;hibernate-mysql&lt;/repository-name&gt;
         &lt;repository-class&gt;org.jvending.util.registry.Hibernate2RepositoryImpl&lt;/repository-class&gt;
         &lt;repository-config&gt;${basedir}/src/test/resources/hibernate2/connections-mysql.xml&lt;/repository-config&gt;  ;
      &lt;/repository&gt;
    &lt;/repositories&gt;

&lt;/registry-config&gt;
</pre>

* connections-hsqldb.xml
<pre>
    &lt;connections&gt;
      &lt;connection&gt;
         &lt;connection-name&gt;peer&lt;/connection-name&gt;
         &lt;connection-config&gt;${basedir}/orm/hsqldb_peer.cfg&lt;/connection-config&gt;
      &lt;/connection&gt;

      &lt;connection&gt;
         &lt;connection-name&gt;catalog&lt;/connection-name&gt;
         &lt;connection-config&gt;${basedir}/orm/hsqldb_catalog.cfg&lt;/connection-config&gt;
      &lt;/connection&gt;

      &lt;connection&gt;
         &lt;connection-name&gt;account&lt;/connection-name&gt;
         &lt;connection-config&gt;${basedir}/orm/hsqldb_user.cfg&lt;/connection-config&gt;
      &lt;/connection&gt;

      &lt;connection&gt;
         &lt;connection-name&gt;metacatalog&lt;/connection-name&gt;
         &lt;connection-config&gt;${basedir}/orm/hsqldb_metacatalog.cfg&lt;/connection-config&gt;
      &lt;/connection&gt;

   &lt;/connections&gt;
 </pre>
 *
 *
 * @author Shane Isbell
 * @since 1.1.0
 *
 */

public final class Hibernate2RepositoryImpl implements Hibernate2Repository, HibernateRepositoryAdaptor {

    private static Logger logger = Logger.getLogger("Hibernate2Repository");

    Map sessionFactories = new HashMap();

    private HibernateRepositoryAdaptee hibernateRepositoryAdaptee = new StandardHibernateRepositoryAdaptee();

    /**
     * @return unmodifiable set of DAOs.
     */
    public Set getDataAccessObjects() {
        return hibernateRepositoryAdaptee.getDataAccessObjects();
    }

    public void setHibernateRepositoryAdaptee(HibernateRepositoryAdaptee hibernateRepositoryAdaptee) {
        logger.info("JV-102-106: Setting the HibernateRepositoryAdaptee: Class = "
                + hibernateRepositoryAdaptee.getClass().getClass().getName());
        this.hibernateRepositoryAdaptee = hibernateRepositoryAdaptee;
    }

    public void load(InputStream inputStream, Hashtable prop) throws IOException {
        hibernateRepositoryAdaptee.setHibernateRepositoryAdaptor(this);
        hibernateRepositoryAdaptee.load(inputStream, prop);
    }

    /**
     * Returns the session factory for the given name.
     *
     * @param name contains the name of the factory
     * @return a session factory for the given name
     * @throws HibernateException if there is a problem obtaining the session factory
     */
    public SessionFactory getSessionFactoryByName(String name) throws HibernateException {
        SessionFactory sessionFactory = (SessionFactory) sessionFactories.get(name);
        if(sessionFactory == null)
            throw new HibernateException("JV-102-001: Session Factory is null: Factory Name = " + name);
        return sessionFactory;
    }

    public Map loadSessionFactory(String connectionName, InputStream inputStream) throws IOException {
        if(connectionName == null) throw new IOException("JV-102-002: Connection Name cannot be null");
        if(inputStream == null)
            throw new IOException("JV-102-003: File cannot be null: Connection Name = " + connectionName);
        try {
            sessionFactories.put(connectionName,
                    new HibernateConfiguration().configure(inputStream).buildSessionFactory());
            logger.info("JV-102-105: Added Session Factory: Connection Name = " + connectionName);
        } catch(HibernateException e) {
            e.printStackTrace();
            throw new IOException("JV-102-004: Had problem loading session factories: Connection Name = "
                    + connectionName);
        }

        return sessionFactories;
    }

    /**
     * Accessor for connection names
     *
     * @return unmodifiable set of Connection Names.
     */
    public Set getConnectionNames() {
        return Collections.unmodifiableSet(sessionFactories.keySet());
    }

    public void setRepositoryRegistry(RepositoryRegistry repositoryRegistry) {
        //don't need this
    }

    private static class HibernateConfiguration extends Configuration {
        public Configuration configure(InputStream inputStream) throws HibernateException {
            return super.doConfigure(inputStream, "");
        }
    }
}
